@@ -10,16 +10,15 @@ use rustc_codegen_ssa::traits::*;
 
 use crate::common;
 use crate::type_of::LayoutLlvmExt;
-use crate::abi::{LlvmType, FnTypeExt};
-use rustc::util::nodemap::FxHashMap;
+use crate::abi::{LlvmType, FnTypeLlvmExt};
+use syntax::ast;
 use rustc::ty::Ty;
-use rustc::ty::layout::TyLayout;
+use rustc::ty::layout::{self, Align, Size, TyLayout};
 use rustc_target::abi::call::{CastTarget, FnType, Reg};
 use rustc_data_structures::small_c_str::SmallCStr;
 use rustc_codegen_ssa::common::TypeKind;
 
 use std::fmt;
-use std::cell::RefCell;
 use std::ptr;
 
 use libc::c_uint;
@@ -52,21 +51,116 @@ impl CodegenCx<'ll, 'tcx> {
                                     els.len() as c_uint, packed as Bool)
         }
     }
-}
 
-impl BaseTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
-    fn type_void(&self) -> &'ll Type {
+    crate fn type_void(&self) -> &'ll Type {
         unsafe {
             llvm::LLVMVoidTypeInContext(self.llcx)
         }
     }
 
-    fn type_metadata(&self) -> &'ll Type {
+    crate fn type_metadata(&self) -> &'ll Type {
         unsafe {
             llvm::LLVMRustMetadataTypeInContext(self.llcx)
         }
     }
 
+    ///x Creates an integer type with the given number of bits, e.g., i24
+    crate fn type_ix(&self, num_bits: u64) -> &'ll Type {
+        unsafe {
+            llvm::LLVMIntTypeInContext(self.llcx, num_bits as c_uint)
+        }
+    }
+
+    crate fn type_x86_mmx(&self) -> &'ll Type {
+        unsafe {
+            llvm::LLVMX86MMXTypeInContext(self.llcx)
+        }
+    }
+
+    crate fn type_vector(&self, ty: &'ll Type, len: u64) -> &'ll Type {
+        unsafe {
+            llvm::LLVMVectorType(ty, len as c_uint)
+        }
+    }
+
+    crate fn func_params_types(&self, ty: &'ll Type) -> Vec<&'ll Type> {
+        unsafe {
+            let n_args = llvm::LLVMCountParamTypes(ty) as usize;
+            let mut args = Vec::with_capacity(n_args);
+            llvm::LLVMGetParamTypes(ty, args.as_mut_ptr());
+            args.set_len(n_args);
+            args
+        }
+    }
+
+    crate fn type_bool(&self) -> &'ll Type {
+        self.type_i8()
+    }
+
+    crate fn type_int_from_ty(&self, t: ast::IntTy) -> &'ll Type {
+        match t {
+            ast::IntTy::Isize => self.type_isize(),
+            ast::IntTy::I8 => self.type_i8(),
+            ast::IntTy::I16 => self.type_i16(),
+            ast::IntTy::I32 => self.type_i32(),
+            ast::IntTy::I64 => self.type_i64(),
+            ast::IntTy::I128 => self.type_i128(),
+        }
+    }
+
+    crate fn type_uint_from_ty(&self, t: ast::UintTy) -> &'ll Type {
+        match t {
+            ast::UintTy::Usize => self.type_isize(),
+            ast::UintTy::U8 => self.type_i8(),
+            ast::UintTy::U16 => self.type_i16(),
+            ast::UintTy::U32 => self.type_i32(),
+            ast::UintTy::U64 => self.type_i64(),
+            ast::UintTy::U128 => self.type_i128(),
+        }
+    }
+
+    crate fn type_float_from_ty(&self, t: ast::FloatTy) -> &'ll Type {
+        match t {
+            ast::FloatTy::F32 => self.type_f32(),
+            ast::FloatTy::F64 => self.type_f64(),
+        }
+    }
+
+    crate fn type_pointee_for_align(&self, align: Align) -> &'ll Type {
+        // FIXME(eddyb) We could find a better approximation if ity.align < align.
+        let ity = layout::Integer::approximate_align(self, align);
+        self.type_from_integer(ity)
+    }
+
+    /// Return a LLVM type that has at most the required alignment,
+    /// and exactly the required size, as a best-effort padding array.
+    crate fn type_padding_filler(&self, size: Size, align: Align) -> &'ll Type {
+        let unit = layout::Integer::approximate_align(self, align);
+        let size = size.bytes();
+        let unit_size = unit.size().bytes();
+        assert_eq!(size % unit_size, 0);
+        self.type_array(self.type_from_integer(unit), size / unit_size)
+    }
+
+    crate fn type_variadic_func(
+        &self,
+        args: &[&'ll Type],
+        ret: &'ll Type
+    ) -> &'ll Type {
+        unsafe {
+            llvm::LLVMFunctionType(ret, args.as_ptr(),
+                                   args.len() as c_uint, True)
+        }
+    }
+
+    crate fn type_array(&self, ty: &'ll Type, len: u64) -> &'ll Type {
+        unsafe {
+            llvm::LLVMRustArrayType(ty, len)
+        }
+    }
+}
+
+impl BaseTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
     fn type_i1(&self) -> &'ll Type {
         unsafe {
             llvm::LLVMInt1TypeInContext(self.llcx)
@@ -104,12 +198,6 @@ impl BaseTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
         }
     }
 
-    fn type_ix(&self, num_bits: u64) -> &'ll Type {
-        unsafe {
-            llvm::LLVMIntTypeInContext(self.llcx, num_bits as c_uint)
-        }
-    }
-
     fn type_isize(&self) -> &'ll Type {
         self.isize_ty
     }
@@ -126,12 +214,6 @@ impl BaseTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
         }
     }
 
-    fn type_x86_mmx(&self) -> &'ll Type {
-        unsafe {
-            llvm::LLVMX86MMXTypeInContext(self.llcx)
-        }
-    }
-
     fn type_func(
         &self,
         args: &[&'ll Type],
@@ -143,17 +225,6 @@ impl BaseTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
         }
     }
 
-    fn type_variadic_func(
-        &self,
-        args: &[&'ll Type],
-        ret: &'ll Type
-    ) -> &'ll Type {
-        unsafe {
-            llvm::LLVMFunctionType(ret, args.as_ptr(),
-                                   args.len() as c_uint, True)
-        }
-    }
-
     fn type_struct(
         &self,
         els: &[&'ll Type],
@@ -166,19 +237,6 @@ impl BaseTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
         }
     }
 
-
-    fn type_array(&self, ty: &'ll Type, len: u64) -> &'ll Type {
-        unsafe {
-            llvm::LLVMRustArrayType(ty, len)
-        }
-    }
-
-    fn type_vector(&self, ty: &'ll Type, len: u64) -> &'ll Type {
-        unsafe {
-            llvm::LLVMVectorType(ty, len as c_uint)
-        }
-    }
-
     fn type_kind(&self, ty: &'ll Type) -> TypeKind {
         unsafe {
             llvm::LLVMRustGetTypeKind(ty).to_generic()
@@ -203,16 +261,6 @@ impl BaseTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
         }
     }
 
-    fn func_params_types(&self, ty: &'ll Type) -> Vec<&'ll Type> {
-        unsafe {
-            let n_args = llvm::LLVMCountParamTypes(ty) as usize;
-            let mut args = Vec::with_capacity(n_args);
-            llvm::LLVMGetParamTypes(ty, args.as_mut_ptr());
-            args.set_len(n_args);
-            args
-        }
-    }
-
     fn float_width(&self, ty: &'ll Type) -> usize {
         match self.type_kind(ty) {
             TypeKind::Float => 32,
@@ -232,10 +280,6 @@ impl BaseTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
     fn val_ty(&self, v: &'ll Value) -> &'ll Type {
         common::val_ty(v)
     }
-
-    fn scalar_lltypes(&self) -> &RefCell<FxHashMap<Ty<'tcx>, Self::Type>> {
-        &self.scalar_lltypes
-    }
 }
 
 impl Type {
@@ -283,7 +327,7 @@ impl LayoutTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
     fn backend_field_index(&self, layout: TyLayout<'tcx>, index: usize) -> u64 {
         layout.llvm_field_index(index)
     }
-    fn scalar_pair_element_backend_type<'a>(
+    fn scalar_pair_element_backend_type(
         &self,
         layout: TyLayout<'tcx>,
         index: usize,
@@ -294,9 +338,6 @@ impl LayoutTypeMethods<'tcx> for CodegenCx<'ll, 'tcx> {
     fn cast_backend_type(&self, ty: &CastTarget) -> &'ll Type {
         ty.llvm_type(self)
     }
-    fn fn_backend_type(&self, ty: &FnType<'tcx, Ty<'tcx>>) -> &'ll Type {
-        ty.llvm_type(self)
-    }
     fn fn_ptr_backend_type(&self, ty: &FnType<'tcx, Ty<'tcx>>) -> &'ll Type {
         ty.ptr_to_llvm_type(self)
     }
