@@ -1,25 +1,13 @@
-// Copyright 2013-2015 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 #![allow(plugin_as_library)]
 #![allow(dead_code)]
 #![allow(unused_variables)]
 #![allow(unused_imports)]
-// aux-build:macro_crate_test.rs
+// aux-build:macro-crate-test.rs
 // ignore-stage1
 
-#![feature(plugin, rustc_attrs)]
-#![plugin(macro_crate_test)]
+#![feature(rustc_attrs)]
 
 #[macro_use]
-#[no_link]
 extern crate macro_crate_test;
 
 // The duplicate macro will create a copy of the item with the given identifier.
