@@ -18,27 +18,28 @@ fn main() {
     // if n > m, it's a type mismatch error.
 
     // n < m
-    let &x = &(&1isize as &T);
-    let &x = &&(&1isize as &T);
-    let &&x = &&(&1isize as &T);
+    let &x = &(&1isize as &dyn T);
+    let &x = &&(&1isize as &dyn T);
+    let &&x = &&(&1isize as &dyn T);
 
     // n == m
-    let &x = &1isize as &T;      //~ ERROR type `&dyn T` cannot be dereferenced
-    let &&x = &(&1isize as &T);  //~ ERROR type `&dyn T` cannot be dereferenced
-    let box x = box 1isize as Box<T>; //~ ERROR type `std::boxed::Box<dyn T>` cannot be dereferenced
+    let &x = &1isize as &dyn T;      //~ ERROR type `&dyn T` cannot be dereferenced
+    let &&x = &(&1isize as &dyn T);  //~ ERROR type `&dyn T` cannot be dereferenced
+    let box x = box 1isize as Box<dyn T>;
+    //~^ ERROR type `std::boxed::Box<dyn T>` cannot be dereferenced
 
     // n > m
-    let &&x = &1isize as &T;
+    let &&x = &1isize as &dyn T;
     //~^ ERROR mismatched types
     //~| expected type `dyn T`
     //~| found type `&_`
     //~| expected trait T, found reference
-    let &&&x = &(&1isize as &T);
+    let &&&x = &(&1isize as &dyn T);
     //~^ ERROR mismatched types
     //~| expected type `dyn T`
     //~| found type `&_`
     //~| expected trait T, found reference
-    let box box x = box 1isize as Box<T>;
+    let box box x = box 1isize as Box<dyn T>;
     //~^ ERROR mismatched types
     //~| expected type `dyn T`
     //~| found type `std::boxed::Box<_>`
