@@ -8,7 +8,7 @@ trait Iterator {
     fn dummy(&self) { }
 }
 
-impl<'a, T> Iterator for &'a mut (Iterator<Item=T> + 'a) {
+impl<'a, T> Iterator for &'a mut (dyn Iterator<Item=T> + 'a) {
     type Item = T;
 }
 
